<?php
// $Id: kwresearch.admin.inc,v 1.1.2.4 2010/08/27 23:19:44 tomdude48 Exp $

/**
 * @file
 * Admin include file.
 */

/**
 * Displays the form for the standard settings tab.
 *
 * @return
 *   array A structured array for use with Forms API.
 */
function kwresearch_admin_settings() {
 
  $apikey = variable_get('kwresearch_apikey','');
  $form['kwresearch_apikey'] = array(
    '#type' => 'textfield',
    '#title' => t('WordTracker API Key'), 
    '#default_value' => $apikey,
    '#description' => t('You need an API key to enable keyword statistics and related phrase lookups. If you have a Wordtracker API key enter it above. To get a full API key, contact !wordtracker_link. You can also use a free !levelten_link to enable demo level access to Wordtracker statitics.',
      array(
        '!wordtracker_link' => l(t('Wordtracker'), 'http://www.wordtracker.com/api', array('attributes' => array('target', 'wordtracker'))),
        '!levelten_link' => l(t('LevelTen SEO API key'), 'http://www.leveltendesign.com/user/0/l10seoapi', array('attributes' => array('target', 'levelten'))),
      )
    ),
  );
  
  if(!module_exists('scribeseo')) {
    $msg = t('You can also use the Scribe SEO API as a source for keyword statitics; !link.',
      array(
        '!link' => l(t('download and install Scribe SEO'), 'http://drupal.org/project/scribeseo', array('attributes' => array('target' => 'drupal'))),
      )
    );
  }
  elseif(!variable_get('scribeseo_apikey', '')) {
    $msg .= t('You can also use the Scribe SEO API as a source for keyword statitics. The Scribe module is installed, however an the API key is missing; !link.',
      array(
        '!link' => l(t('enable the Scribe API key'), 'admin/settings/scribeseo', array('attributes' => array('target' => 'drupal'))),
      )
    );  
  }
  else {
    $msg = t('The Scribe API is enabled as a keyword data source.');
  }
  $form['scribe_apikey'] = array(
    '#type' => 'item',
    '#title' => t('Scribe API Key'), 
    '#value' => $msg,  
  );
  
  if($apikey) {
    $perms = variable_get('kwresearch_api_perms','');
    if(!$perms || ($perms['apikey'] != $apikey)) {
      $perms = kwresearch_api_query_permissions();
      $perms['apikey'] = $apikey;
      //variable_set('kwresearch_api_perms',kwresearch_api_query_permissions());
      //dsm($perms);
    }
    
    $form['kwresearch_acctstatus_hdr'] = array(
      '#type' => 'markup',
      '#value' => '<h3>' . t('Account Permissions') . '</h3>', 
    );
    $statuses = array(
      'get_all_words_popularity' => t('Get all words popularity'),
      'get_embedded_phrase_popularity' => t('Get embedded phrase popularity'),
      'get_exact_phrase_popularity' => t('Get exact phrase popularity'),
      'get_plurals' => t('Get plurals'),
      'get_lateral_keyphrases' => t('Get lateral keyphrases'),
      'get_thesaurus_keyphrases' => t('Get thesaurus keyphrases'),
    );
    foreach($statuses AS $k => $v) {
      $enabled = 'N';
      if($perms[$k]) {
        $limit = $perms[$k]['result_limit'];
        if($limit > 0) {
          $enabled = 'Y (limit '.$limit.')';
        }
      }
      $form['kwresearch_'.$k] = array(
        '#type' => 'item',
        '#title' => $v, 
        '#value' => "Enabled: ".$enabled,
      );
    }
  }
  
  $form['kwresearch_popularity_report_options'] = array(      
    '#type' => 'fieldset',      
    '#title' => t('Popularity report options'),      
    '#collapsible' => TRUE,      
    '#collapsed' => FALSE,    
  );
  $vocabs = taxonomy_get_vocabularies();
  $options = array(0 => t('None'));
  $options2 = array();
  foreach($vocabs AS $vid => $v) {
    if($v->tags) {
      $options[$vid] = $v->name;
      $options2[$vid] = $v->name;
    }
  }
  $form['vocab'] = array(
    '#type' => 'fieldset',
    '#title' => t('Taxonomy settings'),
    '#description' => t('Use the below drop downs to select any vocabularies for tagging nodes with keywords. The Keyword Reserach stores tags in its own tables with meta data and suport for non-nodes. If you want to sync this data with a vocabulary, use the "sync" selection. If you want to store terms in vocabulary independent of Keyword Research\'s data, use the "tag" selection.'),
  );
  $form['vocab']['kwresearch_keyword_sync_vocabulary'] = array(    
    '#type' => 'select',    
    '#title' => t('Page keyword sync vocabulary'),   
    '#options' => $options,
    '#default_value' => variable_get('kwresearch_keyword_sync_vocabulary', 0),     
    '#description' => t('Select a vocabulary to sync with Keyword Research\'s internal page keywords.')     
  );  
  
  $form['vocab']['kwresearch_keyword_tag_vocabulary'] = array(    
    '#type' => 'select',    
    '#title' => t('Page keyword tag vocabulary'),   
    '#options' => $options,
    '#default_value' => variable_get('kwresearch_keyword_tag_vocabulary', 0),     
    '#description' => t('Select a vocabulary for storing keywords independed of Keyword Research\'s page keywords.')     
  );

  $form['vocab']['kwresearch_report_vocabulary'] = array(    
    '#type' => 'checkboxes',    
    '#title' => t('Report vocabulary'),   
    '#options' => $options2,
    '#default_value' => variable_get('kwresearch_report_vocabulary', 0),     
    '#description' => t('Select any vocabulary you would like to attach the keyword reports.')     
  );

  $options = kwresearch_get_report_data_options(); 
  $def_values = kwresearch_get_report_data_options_defaults();
//dsm(variable_get('kwresearch_popularity_report_columns', $def_values));
  $form['kwresearch_popularity_report_options']['kwresearch_popularity_report_data_options'] = array(    
    '#type' => 'checkboxes',    
    '#title' => t('Report data'),   
    '#options' => $options,
    '#default_value' => variable_get('kwresearch_popularity_report_data_options', $def_values),     
    //'#description' => t('')     
  ); 
  
  $options = kwresearch_get_report_links_options(); 
  $def_values = kwresearch_get_report_links_options_defaults();
  $form['kwresearch_popularity_report_options']['kwresearch_popularity_report_links_options'] = array(    
    '#type' => 'checkboxes',    
    '#title' => t('Report links'),   
    '#options' => $options,
    '#default_value' => variable_get('kwresearch_popularity_report_links_options', $def_values),     
    //'#description' => t('')     
  ); 

  return system_settings_form($form);
}

/**
 * Generates site keywords report page
 */
function kwresearch_keywords_list_page($mode = 'all') {
  drupal_add_css(drupal_get_path('module', 'kwresearch') . '/kwresearch.css'); 
  drupal_add_js(drupal_get_path('module', 'kwresearch') . '/kwresearch.js');
  
  // create site keyword data for js
  $site_kw = array();
  $sql = '
    SELECT * FROM {kwresearch_keyword}
    WHERE priority > 0
  ';
  $res = db_query($sql);
  
  while($r = db_fetch_object($res)) {
    $site_kw[$r->keyword] = array(
      'kid' => (int)$r->kid,
      'priority' => (int)$r->priority,
      'value' => (int)$r->value,
      'page_count' =>  (int)$r->page_count,     
    );
  } 
  
  drupal_set_title('Site keywords');
  $filter = kwresearch_build_filter_query();
  $rows = array();

  $output .= drupal_get_form('kwresearch_filter_form');
//dsm($filter);
  $header = array(
    array('data' => t('Keyword'), 'field' => 'k.keyword', 'sort' => 'asc'),
    array('data' => t('Priority'), 'field' => 'k.priority'),
    array('data' => t('Value'), 'field' => 'k.value'),
    array('data' => t('Set by'), 'field' => 'u.name'),
    array('data' => t('Pages'), 'field' => 'k.page_count'),
    array('data' => t('Total daily'), 'field' => 'k.daily_volume'),
    //array('data' => t('Operations')),
  );  
  // note cannot put a line break in pager_query sql statements
  $sql = 'SELECT k.*, u.name AS username FROM {kwresearch_keyword} k LEFT JOIN {users} u ON u.uid = k.uid';
  $tablesort = tablesort_sql($header);
  if (!empty($filter['where'])) {
    $sql = $sql ." WHERE " . (($mode == 'site') ? 'k.priority > 0 AND ' : '') . $filter['where'] . $tablesort;
    $result = pager_query($sql, 100, 0, NULL, $filter['args']);
  }
  else {
    $sql .= (($mode == 'site') ? ' WHERE k.priority' : '');
    $result = pager_query($sql . $tablesort, 100);
  }
  $priorities = kwresearch_get_priority_options();
  while ($r = db_fetch_object($result)) {
    $ops = l(t('edit'),'admin/content/kwresearch/keyword_list/edit/' . $r->kid, array('query' => 'destination=admin/content/kwresearch/keyword_list' . (($mode == 'site') ? '/site' : '')));
    $ops .= ' | ' . l(t('stats'),'admin/content/kwresearch/keyword_report/' . $r->keyword);
    $ops .= ' | ' . l(t('pages'),'admin/content/kwresearch/keyword_pages/' . $r->kid, array('query' => 'destination=admin/content/kwresearch/keyword_list' . (($mode == 'site') ? '/site' : '')));
    
    $rows[] = array('data' =>
      array(
        // Cells
        '<span class="kwresearch_keyword">' . check_plain($r->keyword) . '</span>',
        array('data' => (!$r->priority) ? '-' : $priorities[$r->priority], 'class' => 'site_priority'),
        array('data' => ($r->value == NULL) ? '-' : t('$') . number_format($r->value, 2), 'class' => 'value'),
        array('data' => ($r->username == NULL) ? '-' : l($r->username,'user/' . $r->uid), 'class' => 'user'),
        array('data' => l(number_format($r->page_count),'admin/content/kwresearch/keyword_pages/' . $r->kid, array('query' => 'destination=admin/content/kwresearch/keyword_list' . (($mode == 'site') ? '/site' : ''))), 'class' => 'page_count'),
        array('data' =>  ($r->daily_volume == NULL) ? '-' : number_format($r->daily_volume), 'class' => 'daily_volume'),
        //$ops,
      ),
      // Attributes for tr
      'class' => "kwresearch",
      'id' => "kid-" . $r->kid,
    );
    if(!$site_kw[$r->keyword]) {
      $site_kw[$r->keyword] = array(
        'kid' => (int)$r->kid,
        'priority' => (int)$r->priority,
        'value' => (int)$r->value,
        'page_count' =>  (int)$r->page_count, 
      );
    }
  }

  if (!$rows) {
    $rows[] = array(array('data' => t('No site keywords available.'), 'colspan' => count($header)));
  }  

  $output .= theme('table', $header, $rows, array('id' => 'kwresearch-site-keywords'));
  $output .= theme('pager', NULL, 100, 0);
  $site_priority_options = kwresearch_get_priority_options();
  
  drupal_add_js(
    array('kwresearch' => array(
      'form' => 'admin_keyword_list',
      'analyze_path' => base_path() . 'admin/content/kwresearch/keyword_report/',
      'keyword_edit_path' => base_path() . 'admin/content/kwresearch/keyword_list/edit/',
      'return_destination' => 'admin/content/kwresearch/keyword_list',
      'toggle_site_keyword_callback' => base_path() . 'kwresearch/toggle_site_keyword_js',
      'delete_site_keyword_callback' => base_path() . 'kwresearch/delete_site_keyword_js',
      'module_path' => base_path() . drupal_get_path('module', 'kwresearch'),
      'enable_site_keyword_tool' => user_access('kwresearch admin site keywords'),
      'site_keywords_data' => $site_kw,
      'site_priority_options' => $site_priority_options,
    )), 
  'setting'); 
  return $output;  
}

/**
 * Build query for site keywords report filters based on session.
 */
function kwresearch_build_filter_query() {
  if (empty($_SESSION['kwresearch_overview_filter'])) {
    return;
  }

  $filters = kwresearch_filters();

  // Build query
  $where = $args = array();
//dsm($_SESSION['kwresearch_overview_filter']);
  foreach ($_SESSION['kwresearch_overview_filter'] as $key => $filter) {
    $filter_where = array();
    if($filters[$key]['type'] == 'select') {
      foreach ($filter as $value) {
        $filter_where[] = $filters[$key]['where'];
        $args[] = $value;
      }
    }
    elseif($filter) {
      $filter_where[] = $filters[$key]['where'];
      if($key == 'keyword_includes') {
        $args[] = '%' . $filter  . '%';
      }
      else {
        $args[] = $filter; ;
      }      
    }
    if (!empty($filter_where)) {
      $where[] = '('. implode(' OR ', $filter_where) .')';
    }
  }
  $where = !empty($where) ? implode(' AND ', $where) : '';

  return array(
    'where' => $where,
    'args' => $args,
  );
}

/**
 * Return form for site keyword list filters.
 *
 */
function kwresearch_filter_form() {
  $session = &$_SESSION['kwresearch_overview_filter'];
  $session = is_array($session) ? $session : array();
  $filters = kwresearch_filters();

  $form['filters'] = array(
    '#type' => 'fieldset',
    '#title' => t('Filter keywords'),
    '#theme' => 'dblog_filters',
    '#collapsible' => TRUE,
    '#collapsed' => empty($session),
  );
  foreach ($filters as $key => $filter) {
    if($filter['type'] == 'textfield') {
      $form['filters']['status'][$key] = array(
        '#title' => $filter['title'],
        '#type' => 'textfield',
        '#size' => ($filter['size'])? $filter['size']:8,
      );    
    }
    elseif($filter['type'] == 'select') {
      $form['filters']['status'][$key] = array(
        '#title' => $filter['title'],
        '#type' => 'select',
        '#multiple' => TRUE,
        '#size' => 6,
        '#options' => $filter['options'],
      );
    }
    elseif($filter['type'] == 'markup') {
      $form['filters']['status'][$key] = array(
        '#type' => $filter['type'],
        '#value' => $filter['value'],
      );       
    }
    if (!empty($session[$key])) {
      $form['filters']['status'][$key]['#default_value'] = $session[$key];
    }
  }

  $form['filters']['buttons']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Filter'),
  );
  if (!empty($session)) {
    $form['filters']['buttons']['reset'] = array(
      '#type' => 'submit',
      '#value' => t('Reset')
    );
  }

  return $form;
}

/**
 * Returns filters for site keywords report
 */
function kwresearch_filters() {
  $filters = array();

  $options = kwresearch_get_priority_options();
  $filters['keyword_includes'] = array(
    'title' => t('Includes'),
    'type' => 'textfield',
    'size' => 60,
    'where' => 'k.keyword LIKE "%s"',
  );  
  $filters['priority'] = array(
    'title' => t('Priority'),
    'type' => 'select',
    'where' => 'k.priority = %d',
    'options' => $options,
  );
  
  $filters['value0'] = array(
    'type' => 'markup',
    'value' => '<div id="value-fields-wrapper">'
  );
  $filters['value_min'] = array(
    'title' => t('Min value'),
    'type' => 'textfield',
    'where' => 'k.value >= %d',
  );
  $filters['value_max'] = array(
    'title' => t('Max value'),
    'type' => 'textfield',
    'where' => 'k.value <= %d',
  );
  $filters['value1'] = array(
    'type' => 'markup',
    'value' => '</div>'
  );
  
  $filters['page_count0'] = array(
    'type' => 'markup',
    'value' => '<div id="page_count-fields-wrapper">'
  );
  $filters['page_count_min'] = array(
    'title' => t('Min pages'),
    'type' => 'textfield',
    'where' => 'k.page_count >= %d',
  );
  $filters['page_count_max'] = array(
    'title' => t('Max pages'),
    'type' => 'textfield',
    'where' => 'k.page_count <= %d',
  );
  $filters['page_count1'] = array(
    'type' => 'markup',
    'value' => '</div>'
  );
  
  $filters['daily0'] = array(
    'type' => 'markup',
    'value' => '<div id="daily-fields-wrapper">'
  );
  $filters['daily_min'] = array(
    'title' => t('Min daily volume'),
    'type' => 'textfield',
    'where' => 'k.daily_volume >= %d',
  );
  $filters['daily_max'] = array(
    'title' => t('Max daily volume'),
    'type' => 'textfield',
    'where' => 'k.daily_volume <= %d',
  );
  $filters['daily1'] = array(
    'type' => 'markup',
    'value' => '</div>'
  );

  return $filters;
}

/**
 * Validate result from site keyword report filter form.
 */
function kwresearch_filter_form_validate($form, &$form_state) {
  if ($form_state['values']['op'] == t('Filter') 
    && empty($form_state['values']['keyword_includes'])
    && empty($form_state['values']['priority']) 
    && empty($form_state['values']['value_min'])
    && empty($form_state['values']['value_max'])
    && empty($form_state['values']['daily_min'])
    && empty($form_state['values']['daily_max'])
  ) {
    form_set_error('type', t('You must select something to filter by.'));
  }
}

/**
 * Process result from site keywords report filter form.
 */
function kwresearch_filter_form_submit($form, &$form_state) {
  $op = $form_state['values']['op'];
  $filters = kwresearch_filters();
  switch ($op) {
    case t('Filter'):
      foreach ($filters as $name => $filter) {
        if (isset($form_state['values'][$name])) {
          $_SESSION['kwresearch_overview_filter'][$name] = $form_state['values'][$name];
        }
      }
      break;
    case t('Reset'):
      $_SESSION['kwresearch_overview_filter'] = array();
      break;
  }
  return 'admin/content/kwresearch/site_keywords';
}

/**
 * Generates site keywords edit form page
 * @param int $kid - keyword id if editing keyword data
 */
function kwresearch_keywords_list_edit($kid = 'add') {
  if($kid > 0) {
    $sql = '
      SELECT * 
      FROM {kwresearch_keyword}
      WHERE kid = %d
    ';
    $keyword = db_fetch_object(db_query($sql, $kid));
    drupal_set_title(t('Edit site keyword'));
  }
  if(!$keyword->keyword) {
    drupal_set_title(t('Add site keyword'));
  }  
  return drupal_get_form('kwresearch_site_keywords_edit_form', $keyword);
}

/**
 * Returns site keyword edit form array
 * 
 * @param $form_state
 * @param keyword obj $keyword
 */
function kwresearch_site_keywords_edit_form($form_state, $keyword) {
  if($keyword->kid) {
    $form['kid'] = array(
      '#type' => 'value',
      '#value' => $keyword->kid
    );
    $form['keyword'] = array(
      '#type' => 'item',
      '#title' => t('Keyword'), 
      '#value' => $keyword->keyword,
    );    
  }
  else {
    $form['kid'] = array(
      '#type' => 'value',
      '#value' => 0,
    );
    $form['keyword'] = array(
      '#type' => 'textfield',
      '#title' => t('Keyword'), 
      '#required' => TRUE,
    );  
  }
  
  $options = kwresearch_get_priority_options();
  $form['priority'] = array(
    '#type' => 'select',
    '#title' => t('Priority'), 
    '#options' => $options,
    '#default_value' => $keyword->priority,
    '#description' => t('Use to set how important it is for the site to rank well for this keyword. If unsure, just set to "Standard".'),
  );  

  $form['value'] = array(
    '#type' => 'textfield',
    '#title' => t('Value'),
    '#field_prefix' => t('$'),
    '#default_value' => number_format($keyword->value,2),
    '#description' => t('Use to place a financial value for each visitor who comes to the site using this keyword.'),
  ); 
  
  if($keyword->stats_update) { 
    $form['stats'] = array(      
      '#type' => 'fieldset',      
      '#title' => t('Statistics'),      
      '#collapsible' => TRUE,      
      '#collapsed' => FALSE,    
    );  
    $form['stats']['stats_update'] = array(
      '#type' => 'item',
      '#title' => t('Last updated'), 
      '#value' => format_date($keyword->stats_update),
    );  
    $form['stats']['sdaily_volume'] = array(
      '#type' => 'item',
      '#title' => t('Est. daily search volume'), 
      '#value' => number_format($keyword->daily_volume),
    );  
  }
    
  $form['save'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );
  
  $form['save_update'] = array(
    '#type' => 'submit',
    '#value' => t('Save & update stats'),
  );
  
  return $form;
}

/**
 * Validates site keyword edit form
 * 
 * @param $form
 * @param $form_state
 */
function kwresearch_site_keywords_edit_form_validate($form, &$form_state) {
  if(!is_numeric($form_state['values']['value']) || $form_state['values']['value'] < 0) {
    form_set_error('value', t('Please set a numeric value for the value field greater than 0.'));
  }
}

/**
 * Processes site keyword edit form
 *   saves settings to database
 * 
 * @param $form
 * @param $form_state
 */
function kwresearch_site_keywords_edit_form_submit($form, &$form_state) {
  $keyword_obj = new stdClass();
  $kid = ($form_state['values']['kid']) ? $form_state['values']['kid']:NULL;
  $keyword = strtolower($form_state['values']['keyword']);
  $keyword_obj->priority = $form_state['values']['priority'];
  $keyword_obj->value = (float)$form_state['values']['value'];
  $update_stats = ($form_state['values']['op'] == t('Save & update stats'));
  
  kwresearch_save_site_keyword($keyword, $kid, $keyword_obj, $update_stats);
  
  if(!$kid) {
    drupal_set_message(t('Site keyword @keyword has been added.', 
      array(
        '@keyword' => $keyword
      )),
      'status'
    );
  }
  else {
    drupal_set_message(t('Site keyword @keyword has been updated.', 
      array(
        '@keyword' => $keyword
      )),
      'status'
    );
  }
}

/**
 * Generates site keywords import page
 * 
 * @param int $kid
 */
function kwresearch_site_keywords_import($kid = 'add') {
  drupal_set_title(t('Import site keyword'));
  return drupal_get_form('kwresearch_site_keywords_import_form', $keyword);
}

/**
 * Returns site keywords inport form array
 * 
 * @param $form_state
 * @param str $keyword
 */
function kwresearch_site_keywords_import_form($form_state, $keyword) {
  
  $options = kwresearch_get_priority_options();
  $form['import_data'] = array(
    '#type' => 'textarea',
    '#title' => t('Import data'), 
    '#description' => t('Paste in a list of keywords using the format "keyword,priority,value". Each keyword should be a new line. Priority and value values are optional.'),
  );  

  $form['save'] = array(
    '#type' => 'submit',
    '#value' => t('Add keywords to site list'),
  );
  
  return $form;
}

/**
 * Validates site keywords import form
 * 
 * @param $form
 * @param $form_state
 */
function kwresearch_site_keywords_import_form_validate($form, &$form_state) {
  if(!$form_state['values']['import_data']) {
    form_set_error('import_data', t('Please input data to import.'));
  }
}

/**
 * Processes site keywords import form
 * 
 * @param $form
 * @param $form_state
 */
function kwresearch_site_keywords_import_form_submit($form, &$form_state) {
  GLOBAL $user;
  $lines = explode("\n", $form_state['values']['import_data']);
  $added = 0;
  $updated = 0;
  foreach($lines AS $line) {
    list($keyword, $priority, $value) = explode(",",$line);
    $keyword = drupal_strtolower(trim($keyword));
    if(!$keyword) {
      continue;
    }
    $set = '';
    $fields = '';
    $values = '';
    $format = 0;
    if($priority) {
      $set = ', priority = %d';
      $fields = ", priority";
      $values = ", %d";
      $priority = (int) $priority;
      $format = 1;
    }
    if($value) {
      $set .= ', value = %f';
      $fields .= ", value";
      $values .= ", %f";
      $value = $value;
      $format = 2;
    }
    $sql = '
      UPDATE {kwresearch_keyword}
      SET uid = %d
      ' . $set . '
      WHERE keyword = "%s"
    ';
    if($format == 1) {
      db_query($sql, $user->uid, $priority, $keyword);
    }
    elseif($format == 2) {
      db_query($sql, $user->uid, $priority, $value, $keyword);
    }
    else {
      db_query($sql, $user->uid, $keyword);
    }

    if(db_affected_rows()) {
      $update++;
    }
    else {
      $sql = '
        INSERT INTO {kwresearch_keyword}
        (uid, keyword, word_count' . $fields . ')
        VALUES (%d, "%s", %d' . $values . ')      
      ';
      if($format == 1) {
        db_query($sql, $user->uid, $keyword, str_word_count($keyword), $priority);  
      }
      elseif($format == 2) {
        db_query($sql, $user->uid, $keyword, str_word_count($keyword), $priority, $value);  
      }
      else {
        db_query($sql, $user->uid, $keyword, str_word_count($keyword));  
      }
      $added++;
    }    
  }
  drupal_set_message(t('@added keywords were added and @updated keywords were updated.',
    array(
      '@added' => $added,
      '@updated' => $updated,
    ))
  );
}

/**
 * Generates table of pages assoicated with a keyword
 * 
 * @param int $kid
 */
function kwresearch_keyword_pages_page($kid) {
  drupal_add_css(drupal_get_path('module', 'kwresearch') . '/kwresearch.css'); 
  
  drupal_set_title('Keyword pages');
  
  $keyword_obj = kwresearch_load_site_keyword($kid);
  $output = t('Keyword: @keyword',
    array(
      '@keyword' => $keyword_obj->keyword
    )      
  );
  
  $rows = array();

//dsm($filter);
  $header = array(
    array('data' => t('Page'), 'field' => 'k.page'),
    array('data' => t('Nid'), 'field' => 'pk.nid'),
    array('data' => t('Page priority'), 'field' => 'pk.priority', 'sort' => 'desc'),
    array('data' => t('Operations')),
  );  

  $sql = '
    SELECT pk.* 
    FROM {kwresearch_page_keyword} pk
    WHERE kid = %d
  ';
  $tablesort = tablesort_sql($header);
  $sql = $sql . $tablesort;
  $result = pager_query($sql, 100, 0, NULL, $kid);

  $priorities = kwresearch_get_priority_options();
  while ($r = db_fetch_object($result)) {
    if(!$r->path && $r->nid) {
      $r->path = url('node/' . $r->nid);
    }
    $rows[] = array('data' =>
      array(
        // Cells
        l($r->path,$r->path),
        l($r->nid,'node/' . $r->nid),
        $priorities[$r->priority],
        l(t('keywords'),'admin/content/kwresearch/page_keywords/' . $r->nid, array('query' => 'destination=admin/content/kwresearch/keywords_pages')),
      ),
      // Attributes for tr
      'class' => "kwresearch"
    );
  }

  if (!$rows) {
    $rows[] = array(array('data' => t('No pages associated with this keyword.'), 'colspan' => count($header)));
  }  

  $output .= theme('table', $header, $rows, array('id' => 'kwresearch-site-keywords'));
  $output .= theme('pager', NULL, 100, 0);

  return $output;  
}

/**
 * Generates table of keywords associated with a pages
 * 
 * @param int|str $pid - page id; the nid (int) or path (str) of the page
 */
function kwresearch_page_keywords_page($pid) {
  drupal_add_css(drupal_get_path('module', 'kwresearch') . '/kwresearch.css'); 
  
  drupal_set_title('Page keywords');
  if($pid > 0) {
    $node = node_load($pid);
    $path = 'node/'.$pid;
    $where = 'nid = %d';
  }
  else {
    $path = $pid;
    $where = 'path = "%s"';
  }
  $output = t('Page: !page',
    array(
      '!page' => l($path,$path)
    )      
  );
  
  $rows = array();

//dsm($filter);
  $header = array(
    array('data' => t('Keyword'), 'field' => 'k.keyword'),
    array('data' => t('Page priority'), 'field' => 'pk.priority', 'sort' => 'desc'),
    array('data' => t('Set by'), 'field' => 'up.name'),
    array('data' => t('Site priority'), 'field' => 'k.priority'),
    array('data' => t('Set by'), 'field' => 'u.name'),
    array('data' => t('Value'), 'field' => 'k.value'),
    array('data' => t('Total Daily'), 'field' => 'k.daily_volume'),
    array('data' => t('Operations')),
  );  

  $sql = '
    SELECT pk.priority AS page_priority, pk.uid AS page_uid, k.*, u.name AS username, up.name AS page_username
    FROM {kwresearch_page_keyword} pk
    JOIN {kwresearch_keyword} k ON pk.kid = k.kid
    LEFT JOIN {users} u ON u.uid = k.uid
    LEFT JOIN {users} up ON up.uid = pk.uid
    WHERE ' . $where . '
  ';
  $tablesort = tablesort_sql($header);
  $sql = $sql . $tablesort;
  $result = pager_query($sql, 100, 0, NULL, $pid);

  $priorities = kwresearch_get_priority_options();
  while ($r = db_fetch_object($result)) {
    $rows[] = array('data' =>
      array(
        // Cells
        check_plain($r->keyword),
        $priorities[$r->page_priority],
        ($r->page_username == NULL) ? '-' : l($r->page_username,'user/' . $r->page_uid),
        ($r->priority == NULL) ? '-' : $priorities[$r->priority],
        ($r->username == NULL) ? '-' : l($r->username,'user/' . $r->uid),
        ($r->value == NULL) ? '-' : t('$') . number_format($r->value, 2),
        ($r->daily_volume == NULL) ? '-' : number_format($r->daily_volume),
        l(t('edit'),'admin/content/kwresearch/page_keywords_edit/' . $pid . '/' . $r->kid, array('query' => 'destination=admin/content/kwresearch/page_keywords/' . $pid)),
      ),
      // Attributes for tr
      'class' => "kwresearch"
    );
  }

  if (!$rows) {
    $rows[] = array(array('data' => t('No keywords associated with this page.'), 'colspan' => count($header)));
  }  

  $output .= theme('table', $header, $rows, array('id' => 'kwresearch-site-keywords'));
  $output .= theme('pager', NULL, 100, 0);
  
  $output .= l(t('Add page keyword'),'admin/content/kwresearch/page_keywords_edit/' . $pid, array('query' => 'destination=admin/content/kwresearch/page_keywords/' . $pid));

  return $output;  
}

/**
 * Generages page keyword edit form
 * 
 * @param int|str $pid - page id; the nid (int) or path (str) of the page
 * @param int $kid - keyword id
 */
function kwresearch_page_keywords_edit($pid, $kid = 'add') {
  if($kid > 0) {
    $sql = '
      SELECT * 
      FROM {kwresearch_keyword}
      WHERE kid = %d
    ';
    $keyword = db_fetch_object(db_query($sql, $kid));
    drupal_set_title(t('Edit page keyword'));
  }
  if(!$keyword->keyword) {
    drupal_set_title(t('Add page keyword'));
  }  
  $output .= drupal_get_form('kwresearch_page_keywords_edit_form', $pid, $keyword);
  return $output;
}

/**
 * Returns page keyword edit form
 * 
 * @param arr $form_state
 * @param int|str $pid - page id; the nid (int) or path (str) of the page
 * @param str $keyword
 */

function kwresearch_page_keywords_edit_form($form_state, $pid, $keyword) {
  $url = url(($pid > 0)?'node/' . $pid:$pid);
  if(!$url) {
    drupal_set_message(t('Invalid page entered.'));
    return;
  }
  
  $page_keyword = kwresearch_load_page_keyword($pid, $keyword->kid);

  $form['page'] = array(
      '#title' => t('Page'),
      '#type' => 'item',
      '#value' =>  l($url, $url),
  );
  $form['pid'] = array(
    '#type' => 'value',
    '#value' => $pid
  );
  if($keyword->kid) {
    $form['kid'] = array(
      '#type' => 'value',
      '#value' => $keyword->kid
    );
    $form['keyword'] = array(
      '#type' => 'item',
      '#title' => t('Keyword'), 
      '#value' => $keyword->keyword,
    );    
  }
  else {
    $form['kid'] = array(
      '#type' => 'value',
      '#value' => 0,
    );
    $form['keyword'] = array(
      '#type' => 'textfield',
      '#title' => t('Keyword'), 
      '#required' => TRUE,
    );  
  }
  
  $options = kwresearch_get_priority_options();
  $form['page_priority'] = array(
    '#type' => 'select',
    '#title' => t('Priority'), 
    '#options' => $options,
    '#default_value' => $page_keyword->priority,
    '#description' => t('Use to set how important it is for the site to rank well for this keyword. If unsure, just set to "Standard".'),
  ); 
    
  $form['save'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );
  
  $form['save_update'] = array(
    '#type' => 'submit',
    '#value' => t('Save & update stats'),
  );
  
  return $form;
}

/**
 * Validates the page keyword edit form
 * 
 * @param $form
 * @param $form_state
 */
function kwresearch_page_keywords_edit_form_validate($form, &$form_state) {
  if($form_state['values']['value'] && (!is_numeric($form_state['values']['value']) || $form_state['values']['value'] < 0)) {
    form_set_error('value', t('Please set a numeric value for the value field greater than 0.'));
  }
}

/**
 * Processes the page keyword edit form
 * 
 * @param $form
 * @param $form_state
 */
function kwresearch_page_keywords_edit_form_submit($form, &$form_state) {
  $kid = $form_state['values']['kid'];
  $keyword = strtolower($form_state['values']['keyword']);
  $pid = $form_state['values']['pid'];
  $page_keyword_obj = new stdClass();
  $page_keyword_obj->priority = $form_state['values']['page_priority'];

  kwresearch_save_page_keyword($pid, $keyword, $kid, $page_keyword_obj);
  
  drupal_set_message(t('Site keyword @keyword has been updated.', 
    array(
      '@keyword' => $keyword
    )),
    'status'
  );
}

